/* crypto/evp/e_indect.c -*- mode:C; c-file-style: "eay" -*- */
/* ====================================================================
 * Copyright (c) 2006 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include <openssl/opensslconf.h>
#ifndef OPENSSL_NO_INDECT
#include <openssl/evp.h>
#include <openssl/err.h>
#include <string.h>
#include <assert.h>
#include <openssl/indect.h>
#include "evp_locl.h"

static int indect_init_key(EVP_CIPHER_CTX *ctx, const unsigned char *key,
	const unsigned char *iv, int enc);

/* Indect subkey Structure */
typedef struct
	{
	INDECT_KEY ks;
	} EVP_INDECT_KEY;

/* Attribute operation for Indect */
#define data(ctx)	EVP_C_DATA(EVP_INDECT_KEY,ctx)

IMPLEMENT_BLOCK_CIPHER(indect_128, ks, Indect, EVP_INDECT_KEY,
	NID_indect_128, 16, 16, 16, 128,
	0, indect_init_key, NULL, 
	EVP_CIPHER_set_asn1_iv,
	EVP_CIPHER_get_asn1_iv,
	NULL)
IMPLEMENT_BLOCK_CIPHER(indect_192, ks, Indect, EVP_INDECT_KEY,
	NID_indect_192, 16, 24, 16, 128,
	0, indect_init_key, NULL, 
	EVP_CIPHER_set_asn1_iv,
	EVP_CIPHER_get_asn1_iv,
	NULL)
IMPLEMENT_BLOCK_CIPHER(indect_320, ks, Indect, EVP_INDECT_KEY,
	NID_indect_320, 16, 40, 16, 128,
	0, indect_init_key, NULL, 
	EVP_CIPHER_set_asn1_iv,
	EVP_CIPHER_get_asn1_iv,
	NULL)

#define IMPLEMENT_INDECT_CFBR(ksize,cbits)	IMPLEMENT_CFBR(indect,Indect,EVP_INDECT_KEY,ks,ksize,cbits,32,0)

IMPLEMENT_INDECT_CFBR(128,1)
IMPLEMENT_INDECT_CFBR(192,1)
IMPLEMENT_INDECT_CFBR(320,1)

IMPLEMENT_INDECT_CFBR(128,8)
IMPLEMENT_INDECT_CFBR(192,8)
IMPLEMENT_INDECT_CFBR(320,8)



/* The subkey for Indect is generated. */ 
static int indect_init_key(EVP_CIPHER_CTX *ctx, const unsigned char *key,
	const unsigned char *iv, int enc)
	{
	int ret;

	if ((ctx->cipher->flags & EVP_CIPH_MODE) == EVP_CIPH_CFB_MODE
	    || (ctx->cipher->flags & EVP_CIPH_MODE) == EVP_CIPH_OFB_MODE
	    || enc) 
		ret=Indect_set_encrypt_key(key, ctx->key_len * 8, ctx->cipher_data);
	else
		ret=Indect_set_decrypt_key(key, ctx->key_len * 8, ctx->cipher_data);
            
	if(ret < 0)
		{
		EVPerr(EVP_F_INDECT_INIT_KEY,EVP_R_INDECT_KEY_SETUP_FAILED);
		return 0;
		}

	return 1;
	}

#else

# ifdef PEDANTIC
static void *dummy=&dummy;
# endif

#endif
