/* crypto/indect/indect.c -*- mode:C; c-file-style: "eay" -*- */
/* ====================================================================
 * Copyright (c) 2012 Piotr Jurkiewicz
 *
 * This work has been funded by the European Commission FP7 project
 * INDECT under grant agreement No. 218086.
 *
 * The Indect Block Cipher code is licensed pursuant to the OpenSSL open
 * source license provided below.
 *
 */
/* ====================================================================
 * Copyright (c) 2006 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */


#include <string.h>
#include <stdlib.h>

#include "indect.h"
#include "ibc_locl.h"

static const unsigned char aes_fwd[256] = 
{
   0x63, 0x7C, 0x77, 0x7B, 0xF2, 0x6B, 0x6F, 0xC5, 0x30, 0x01, 0x67, 0x2B, 0xFE, 0xD7, 0xAB, 0x76,
   0xCA, 0x82, 0xC9, 0x7D, 0xFA, 0x59, 0x47, 0xF0, 0xAD, 0xD4, 0xA2, 0xAF, 0x9C, 0xA4, 0x72, 0xC0,
   0xB7, 0xFD, 0x93, 0x26, 0x36, 0x3F, 0xF7, 0xCC, 0x34, 0xA5, 0xE5, 0xF1, 0x71, 0xD8, 0x31, 0x15,
   0x04, 0xC7, 0x23, 0xC3, 0x18, 0x96, 0x05, 0x9A, 0x07, 0x12, 0x80, 0xE2, 0xEB, 0x27, 0xB2, 0x75,
   0x09, 0x83, 0x2C, 0x1A, 0x1B, 0x6E, 0x5A, 0xA0, 0x52, 0x3B, 0xD6, 0xB3, 0x29, 0xE3, 0x2F, 0x84,
   0x53, 0xD1, 0x00, 0xED, 0x20, 0xFC, 0xB1, 0x5B, 0x6A, 0xCB, 0xBE, 0x39, 0x4A, 0x4C, 0x58, 0xCF,
   0xD0, 0xEF, 0xAA, 0xFB, 0x43, 0x4D, 0x33, 0x85, 0x45, 0xF9, 0x02, 0x7F, 0x50, 0x3C, 0x9F, 0xA8,
   0x51, 0xA3, 0x40, 0x8F, 0x92, 0x9D, 0x38, 0xF5, 0xBC, 0xB6, 0xDA, 0x21, 0x10, 0xFF, 0xF3, 0xD2,
   0xCD, 0x0C, 0x13, 0xEC, 0x5F, 0x97, 0x44, 0x17, 0xC4, 0xA7, 0x7E, 0x3D, 0x64, 0x5D, 0x19, 0x73,
   0x60, 0x81, 0x4F, 0xDC, 0x22, 0x2A, 0x90, 0x88, 0x46, 0xEE, 0xB8, 0x14, 0xDE, 0x5E, 0x0B, 0xDB,
   0xE0, 0x32, 0x3A, 0x0A, 0x49, 0x06, 0x24, 0x5C, 0xC2, 0xD3, 0xAC, 0x62, 0x91, 0x95, 0xE4, 0x79,
   0xE7, 0xC8, 0x37, 0x6D, 0x8D, 0xD5, 0x4E, 0xA9, 0x6C, 0x56, 0xF4, 0xEA, 0x65, 0x7A, 0xAE, 0x08,
   0xBA, 0x78, 0x25, 0x2E, 0x1C, 0xA6, 0xB4, 0xC6, 0xE8, 0xDD, 0x74, 0x1F, 0x4B, 0xBD, 0x8B, 0x8A,
   0x70, 0x3E, 0xB5, 0x66, 0x48, 0x03, 0xF6, 0x0E, 0x61, 0x35, 0x57, 0xB9, 0x86, 0xC1, 0x1D, 0x9E,
   0xE1, 0xF8, 0x98, 0x11, 0x69, 0xD9, 0x8E, 0x94, 0x9B, 0x1E, 0x87, 0xE9, 0xCE, 0x55, 0x28, 0xDF,
   0x8C, 0xA1, 0x89, 0x0D, 0xBF, 0xE6, 0x42, 0x68, 0x41, 0x99, 0x2D, 0x0F, 0xB0, 0x54, 0xBB, 0x16
};

static const unsigned char aes_inv[256] = 
{
   0x52, 0x09, 0x6A, 0xD5, 0x30, 0x36, 0xA5, 0x38, 0xBF, 0x40, 0xA3, 0x9E, 0x81, 0xF3, 0xD7, 0xFB,
   0x7C, 0xE3, 0x39, 0x82, 0x9B, 0x2F, 0xFF, 0x87, 0x34, 0x8E, 0x43, 0x44, 0xC4, 0xDE, 0xE9, 0xCB,
   0x54, 0x7B, 0x94, 0x32, 0xA6, 0xC2, 0x23, 0x3D, 0xEE, 0x4C, 0x95, 0x0B, 0x42, 0xFA, 0xC3, 0x4E,
   0x08, 0x2E, 0xA1, 0x66, 0x28, 0xD9, 0x24, 0xB2, 0x76, 0x5B, 0xA2, 0x49, 0x6D, 0x8B, 0xD1, 0x25,
   0x72, 0xF8, 0xF6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xD4, 0xA4, 0x5C, 0xCC, 0x5D, 0x65, 0xB6, 0x92,
   0x6C, 0x70, 0x48, 0x50, 0xFD, 0xED, 0xB9, 0xDA, 0x5E, 0x15, 0x46, 0x57, 0xA7, 0x8D, 0x9D, 0x84,
   0x90, 0xD8, 0xAB, 0x00, 0x8C, 0xBC, 0xD3, 0x0A, 0xF7, 0xE4, 0x58, 0x05, 0xB8, 0xB3, 0x45, 0x06,
   0xD0, 0x2C, 0x1E, 0x8F, 0xCA, 0x3F, 0x0F, 0x02, 0xC1, 0xAF, 0xBD, 0x03, 0x01, 0x13, 0x8A, 0x6B,
   0x3A, 0x91, 0x11, 0x41, 0x4F, 0x67, 0xDC, 0xEA, 0x97, 0xF2, 0xCF, 0xCE, 0xF0, 0xB4, 0xE6, 0x73,
   0x96, 0xAC, 0x74, 0x22, 0xE7, 0xAD, 0x35, 0x85, 0xE2, 0xF9, 0x37, 0xE8, 0x1C, 0x75, 0xDF, 0x6E,
   0x47, 0xF1, 0x1A, 0x71, 0x1D, 0x29, 0xC5, 0x89, 0x6F, 0xB7, 0x62, 0x0E, 0xAA, 0x18, 0xBE, 0x1B,
   0xFC, 0x56, 0x3E, 0x4B, 0xC6, 0xD2, 0x79, 0x20, 0x9A, 0xDB, 0xC0, 0xFE, 0x78, 0xCD, 0x5A, 0xF4,
   0x1F, 0xDD, 0xA8, 0x33, 0x88, 0x07, 0xC7, 0x31, 0xB1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xEC, 0x5F,
   0x60, 0x51, 0x7F, 0xA9, 0x19, 0xB5, 0x4A, 0x0D, 0x2D, 0xE5, 0x7A, 0x9F, 0x93, 0xC9, 0x9C, 0xEF,
   0xA0, 0xE0, 0x3B, 0x4D, 0xAE, 0x2A, 0xF5, 0xB0, 0xC8, 0xEB, 0xBB, 0x3C, 0x83, 0x53, 0x99, 0x61,
   0x17, 0x2B, 0x04, 0x7E, 0xBA, 0x77, 0xD6, 0x26, 0xE1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0C, 0x7D
};

void update_invalidlc_table(unsigned char chosenlc, unsigned char invalidlc_table[])
{
	unsigned char currentlc = 0;
	
	while (1) {
		if (invalidlc_table[currentlc] == 1) invalidlc_table[chosenlc^currentlc] = 1;
		if (currentlc == 255) break;
		++currentlc;
	}
	
	return;
}

unsigned char chooselc(unsigned char mappedlc, unsigned char invalidlc_table[])
{
	unsigned char chosenlc;
	
	unsigned char currentlc = 0;
	int currentvalidlc = -1;
	
	while (1) {
		if (invalidlc_table[currentlc] == 0) ++currentvalidlc;
		if (currentvalidlc == mappedlc) {chosenlc = currentlc; break;}
		++currentlc;
	}
	
	update_invalidlc_table(chosenlc, invalidlc_table);
	return chosenlc;
	
}

unsigned char parity(unsigned char byte)
{
	byte ^= byte >> 4;
	byte ^= byte >> 2;
	byte ^= byte >> 1;
	return byte & 1;
}


void indect_setup(const unsigned char *key, const int bits, unsigned char sbox[][256], unsigned char ptab[][2], const int enc)
{
	int sboxi;
	int sboxn;

	unsigned char pbox[128];
	
	sboxn = bits/64;
	
	for (sboxi = 0; sboxi < sboxn; sboxi++) {
		
		unsigned char invalidlc_table[256] = {
		1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		
		unsigned char chosenlc[8];
		unsigned int aes_inverted;
		
		unsigned char rawkeylc;
		unsigned char mappedlc;
		
		rawkeylc = key[8*sboxi+0];
		mappedlc = (rawkeylc*255)/256;
		chosenlc[0] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+1];
		mappedlc = (rawkeylc*254)/256;
		chosenlc[1] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+2];
		mappedlc = (rawkeylc*252)/256;
		chosenlc[2] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+3];
		mappedlc = (rawkeylc*248)/256;
		chosenlc[3] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+4];
		mappedlc = (rawkeylc*240)/256;
		chosenlc[4] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+5];
		mappedlc = (rawkeylc*224)/256;
		chosenlc[5] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+6];
		mappedlc = (rawkeylc*192)/256;
		chosenlc[6] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+7];
		mappedlc = (rawkeylc*128)/256;
		chosenlc[7] = chooselc(mappedlc, invalidlc_table);

		rawkeylc = key[8*sboxi+7];
		aes_inverted = rawkeylc & 1;
		
		if (!aes_inverted) {
		
			int n;

			for (n = 0; n < 256; n++) {
				
				sbox[sboxi][n] = 0x00;
				
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[0]) << 0;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[1]) << 1;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[2]) << 2;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[3]) << 3;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[4]) << 4;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[5]) << 5;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[6]) << 6;
				sbox[sboxi][n] ^= parity(aes_fwd[n] & chosenlc[7]) << 7;
				
			}
		}
		
		if (aes_inverted) {
		
			int n;

			for (n = 0; n < 256; n++) {
				
				sbox[sboxi][n] = 0x00;
				
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[0]) << 0;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[1]) << 1;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[2]) << 2;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[3]) << 3;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[4]) << 4;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[5]) << 5;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[6]) << 6;
				sbox[sboxi][n] ^= parity(aes_inv[n] & chosenlc[7]) << 7;
				
			}
		}
		
	}
	
	if (!enc)
	{
		int n;
		unsigned char isbox[INDECT_SBOXES_MAXNR][256];	
		for (sboxi = 0; sboxi < sboxn-1; sboxi++) for (n = 0; n < 256; n++) isbox[sboxi][sbox[sboxi][n]] = n;
		for (sboxi = 0; sboxi < sboxn-1; sboxi++) for (n = 0; n < 256; n++)  sbox[sboxi][n] = isbox[sboxi][n];
	}
	
	int bit;
	int newbit;

	newbit = 0;

	for (bit=0; bit < 256; bit++) if (sbox[sboxn-1][bit] < 128)
	{
		pbox[newbit] = sbox[sboxn-1][bit];
		newbit += 1;
	}

	if (!enc)
	{
		int n;
		unsigned char ipbox[128];	
		for (n = 0; n < 128; n++) ipbox[pbox[n]] = n;
		for (n = 0; n < 128; n++) pbox[n] = ipbox[n];
	}

	for (bit=0; bit < 128; bit++)
	{
		ptab[bit][0] = pbox[bit]/8;
		ptab[bit][1] = pbox[bit]%8;
	}

	return;
}

      
      
void indect_encrypt128(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2])
{
	unsigned char s[16];
	unsigned char t[16];
	
	unsigned int bit;
	unsigned int round;
	
	memset(s,0,16);
	for (bit=0; bit < 128; bit++) s[ptab[bit][0]] |=  ((in[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
		
	for (round=0; round < 8; round++) {
		
		t[0] = sbox[0][s[0]];
		t[1] = sbox[0][s[1]];
		t[2] = sbox[0][s[2]];
		t[3] = sbox[0][s[3]];
		t[4] = sbox[0][s[4]];
		t[5] = sbox[0][s[5]];
		t[6] = sbox[0][s[6]];
		t[7] = sbox[0][s[7]];
		t[8] = sbox[0][s[8]];
		t[9] = sbox[0][s[9]];
		t[10] = sbox[0][s[10]];
		t[11] = sbox[0][s[11]];
		t[12] = sbox[0][s[12]];
		t[13] = sbox[0][s[13]];
		t[14] = sbox[0][s[14]];
		t[15] = sbox[0][s[15]];

		memset(s,0,16);
		for (bit=0; bit < 128; bit++) s[ptab[bit][0]] |=  ((t[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
	}
	
	memcpy(out,s,16);
	return;
}

void indect_decrypt128(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2])
{
	unsigned char s[16];
	unsigned char t[16];
	
	unsigned int bit;
	unsigned int round;
	
	memcpy(s,in,16);
		
	for (round=0; round < 8; round++) {
	
		memset(t,0,16);
		for (bit=0; bit < 128; bit++) t[ptab[bit][0]] |=  ((s[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
		
		s[0] = sbox[0][t[0]];
		s[1] = sbox[0][t[1]];
		s[2] = sbox[0][t[2]];
		s[3] = sbox[0][t[3]];
		s[4] = sbox[0][t[4]];
		s[5] = sbox[0][t[5]];
		s[6] = sbox[0][t[6]];
		s[7] = sbox[0][t[7]];
		s[8] = sbox[0][t[8]];
		s[9] = sbox[0][t[9]];
		s[10] = sbox[0][t[10]];
		s[11] = sbox[0][t[11]];
		s[12] = sbox[0][t[12]];
		s[13] = sbox[0][t[13]];
		s[14] = sbox[0][t[14]];
		s[15] = sbox[0][t[15]];
	}
	
	memset(out,0,16);
	for (bit=0; bit < 128; bit++) out[ptab[bit][0]] |=  ((s[bit/8] << (bit%8)) & 128) >> ptab[bit][1];

	return;
}
      
void indect_encrypt192(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2])
{
	unsigned char s[16];
	unsigned char t[16];
	
	unsigned int bit;
	unsigned int round;
	
	memset(s,0,16);
	for (bit=0; bit < 128; bit++) s[ptab[bit][0]] |=  ((in[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
		
	for (round=0; round < 10; round++) {
		
		t[0] = sbox[0][s[0]];
		t[1] = sbox[0][s[1]];
		t[2] = sbox[0][s[2]];
		t[3] = sbox[0][s[3]];
		t[4] = sbox[0][s[4]];
		t[5] = sbox[0][s[5]];
		t[6] = sbox[0][s[6]];
		t[7] = sbox[0][s[7]];
		t[8] = sbox[1][s[8]];
		t[9] = sbox[1][s[9]];
		t[10] = sbox[1][s[10]];
		t[11] = sbox[1][s[11]];
		t[12] = sbox[1][s[12]];
		t[13] = sbox[1][s[13]];
		t[14] = sbox[1][s[14]];
		t[15] = sbox[1][s[15]];

		memset(s,0,16);
		for (bit=0; bit < 128; bit++) s[ptab[bit][0]] |=  ((t[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
	}
	
	memcpy(out,s,16);
	return;
}

void indect_decrypt192(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2])
{
	unsigned char s[16];
	unsigned char t[16];
	
	unsigned int bit;
	unsigned int round;
	
	memcpy(s,in,16);
		
	for (round=0; round < 10; round++) {
	
		memset(t,0,16);
		for (bit=0; bit < 128; bit++) t[ptab[bit][0]] |=  ((s[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
		
		s[0] = sbox[0][t[0]];
		s[1] = sbox[0][t[1]];
		s[2] = sbox[0][t[2]];
		s[3] = sbox[0][t[3]];
		s[4] = sbox[0][t[4]];
		s[5] = sbox[0][t[5]];
		s[6] = sbox[0][t[6]];
		s[7] = sbox[0][t[7]];
		s[8] = sbox[1][t[8]];
		s[9] = sbox[1][t[9]];
		s[10] = sbox[1][t[10]];
		s[11] = sbox[1][t[11]];
		s[12] = sbox[1][t[12]];
		s[13] = sbox[1][t[13]];
		s[14] = sbox[1][t[14]];
		s[15] = sbox[1][t[15]];
	}
	
	memset(out,0,16);
	for (bit=0; bit < 128; bit++) out[ptab[bit][0]] |=  ((s[bit/8] << (bit%8)) & 128) >> ptab[bit][1];

	return;
}

void indect_encrypt320(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2])
{
	unsigned char s[16];
	unsigned char t[16];
	
	unsigned int bit;
	unsigned int round;
	
	memset(s,0,16);
	for (bit=0; bit < 128; bit++) s[ptab[bit][0]] |=  ((in[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
		
	for (round=0; round < 12; round++) {
		
		t[0] = sbox[0][s[0]];
		t[1] = sbox[0][s[1]];
		t[2] = sbox[0][s[2]];
		t[3] = sbox[0][s[3]];
		t[4] = sbox[1][s[4]];
		t[5] = sbox[1][s[5]];
		t[6] = sbox[1][s[6]];
		t[7] = sbox[1][s[7]];
		t[8] = sbox[2][s[8]];
		t[9] = sbox[2][s[9]];
		t[10] = sbox[2][s[10]];
		t[11] = sbox[2][s[11]];
		t[12] = sbox[3][s[12]];
		t[13] = sbox[3][s[13]];
		t[14] = sbox[3][s[14]];
		t[15] = sbox[3][s[15]];

		memset(s,0,16);
		for (bit=0; bit < 128; bit++) s[ptab[bit][0]] |=  ((t[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
	}
	
	memcpy(out,s,16);
	return;
}

void indect_decrypt320(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2])
{
	unsigned char s[16];
	unsigned char t[16];
	
	unsigned int bit;
	unsigned int round;
	
	memcpy(s,in,16);
		
	for (round=0; round < 12; round++) {
	
		memset(t,0,16);
		for (bit=0; bit < 128; bit++) t[ptab[bit][0]] |=  ((s[bit/8] << (bit%8)) & 128) >> ptab[bit][1];
		
		s[0] = sbox[0][t[0]];
		s[1] = sbox[0][t[1]];
		s[2] = sbox[0][t[2]];
		s[3] = sbox[0][t[3]];
		s[4] = sbox[1][t[4]];
		s[5] = sbox[1][t[5]];
		s[6] = sbox[1][t[6]];
		s[7] = sbox[1][t[7]];
		s[8] = sbox[2][t[8]];
		s[9] = sbox[2][t[9]];
		s[10] = sbox[2][t[10]];
		s[11] = sbox[2][t[11]];
		s[12] = sbox[3][t[12]];
		s[13] = sbox[3][t[13]];
		s[14] = sbox[3][t[14]];
		s[15] = sbox[3][t[15]];
	}
	
	memset(out,0,16);
	for (bit=0; bit < 128; bit++) out[ptab[bit][0]] |=  ((s[bit/8] << (bit%8)) & 128) >> ptab[bit][1];

	return;
}

