/* crypto/indect/indect.h -*- mode:C; c-file-style: "eay" -*- */
/* ====================================================================
 * Copyright (c) 2006 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#ifndef HEADER_INDECT_H
#define HEADER_INDECT_H

#include <openssl/opensslconf.h>

#ifdef OPENSSL_NO_INDECT
#error INDECT is disabled.
#endif

#define INDECT_ENCRYPT	1
#define INDECT_DECRYPT	0

/* Because array size can't be a const in C, the following two are macros.
   Both sizes are in bytes. */
#define INDECT_BLOCK_SIZE 16
#define INDECT_MAX_KEY_SIZE 576 // in bits
#define INDECT_SBOXES_MAXNR (INDECT_MAX_KEY_SIZE / 64)   




#ifdef  __cplusplus
extern "C" {
#endif

/* This should be a hidden type, but EVP requires that the size be known */



struct indect_key_st 
	{
	unsigned char sbox[INDECT_SBOXES_MAXNR][256];
	unsigned char ptab[128][2];
	int bitLength;
	void (*enc)(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2]);
	void (*dec)(const unsigned char *in, unsigned char *out, const unsigned char sbox[][256], const unsigned char ptab[][2]);
	};
typedef struct indect_key_st INDECT_KEY;

int Indect_set_encrypt_key(const unsigned char *userKey, const int bits,
	INDECT_KEY *key);
int Indect_set_decrypt_key(const unsigned char *userKey, const int bits,
	INDECT_KEY *key);

void Indect_encrypt(const unsigned char *in, unsigned char *out,
	const INDECT_KEY *key);
void Indect_decrypt(const unsigned char *in, unsigned char *out,
	const INDECT_KEY *key);

void Indect_ecb_encrypt(const unsigned char *in, unsigned char *out,
	const INDECT_KEY *key, const int enc);
void Indect_cbc_encrypt(const unsigned char *in, unsigned char *out,
	const unsigned long length, const INDECT_KEY *key,
	unsigned char *ivec, const int enc);
void Indect_cfb128_encrypt(const unsigned char *in, unsigned char *out,
	const unsigned long length, const INDECT_KEY *key,
	unsigned char *ivec, int *num, const int enc);
void Indect_cfb1_encrypt(const unsigned char *in, unsigned char *out,
	const unsigned long length, const INDECT_KEY *key,
	unsigned char *ivec, int *num, const int enc);
void Indect_cfb8_encrypt(const unsigned char *in, unsigned char *out,
	const unsigned long length, const INDECT_KEY *key,
	unsigned char *ivec, int *num, const int enc);
void Indect_cfbr_encrypt_block(const unsigned char *in,unsigned char *out,
	const int nbits,const INDECT_KEY *key,
	unsigned char *ivec,const int enc);
void Indect_ofb128_encrypt(const unsigned char *in, unsigned char *out,
	const unsigned long length, const INDECT_KEY *key,
	unsigned char *ivec, int *num);
void Indect_ctr128_encrypt(const unsigned char *in, unsigned char *out,
	const unsigned long length, const INDECT_KEY *key,
	unsigned char ivec[INDECT_BLOCK_SIZE],
	unsigned char ecount_buf[INDECT_BLOCK_SIZE],
	unsigned int *num);

#ifdef  __cplusplus
}
#endif

#endif /* !HEADER_Indect_H */

